//----------------------------------------------------------------------------
//~Author: Fabio Arciniegas A.
//~Name: Indent
//~Description: Indent (com.faa.sax.util.Indent) is a Java SAX program to
// format an XML document. 
// It comes with:
//
//     - Command line version 
//     - dos batch wrappers to mimic alias behavior under DOS
//
//Version $Id: Indent.java,v 1.3 2001/01/28 16:05:49 Default Exp $
//----------------------------------------------------------------------------
//             This code has been formated to fit in 80 columns
//----------------------------------------------------------------------------
package com.faa.sax.util;


// Classes
import java.lang.           Integer;
import java.io.             PrintStream;
import java.io.             FileOutputStream;
import org.xml.sax.helpers. DefaultHandler;


// Interfaces
import org.xml.sax. XMLReader;



  /** 
Indent (com.faa.sax.util.Indent) is a Java SAX program to format an
XML document. 
<h2>Syntax</h2>

<b>java com.faa.sax.util.Indent [options] [inputFile [outputFile]]</b>

<h2>Options</h2>
<pre>
	-t=X indent elements using X tabs. Overrides -s. Default: off.
	-s=X indent elements using X spaces. Default: -s=3
	-f=X auto-fill text at column X. Default: off.
	-a=X number of attributes per line. Default: -a=1
	-l+X align running line text to start or end of
	    parent element start tag (plus and optional X offset).
		     Default: -l+1
        -l1 align first line of text too. Default: off
	-parser=X. sets parser to class X.Default: 
	            org.apache.xerces.parsers.SAXParser
</pre>
*/
public class Indent
{

// ******************************************         Constructors / Destructors

  /** 
      Create the parser and start processing
   */
 
    public static void startIndent()
    {
	try
	{
	    PrintStream outFile = out != null 
                                ? new PrintStream(new FileOutputStream(out)) 
                                : System.out;
	    IndentHandler handler = new IndentHandler(tabs != 0,
						      spaces == 0 ?tabs:spaces,
						      fill,
						      attributes,
						      lineOffset,
						      alignFirstLine,
						      outFile);
	    XMLReader parser = (XMLReader)
                               Class.forName(parserName).newInstance();
	    parser.setFeature("http://xml.org/sax/features/validation",true);
	    parser.setFeature("http://xml.org/sax/features/namespaces",true);


	    parser.setContentHandler(handler);
	    // Everything is ready. Parse and receive the fruits.
	    parser.parse(in);
	}
	catch(Exception e)
	{
	    e.printStackTrace(System.err);
	}
    }
/**
   Deal with application parameters. 
   Create the sax processor and start the parsing process
 */

    public static void main(String args[])
    {
	for(int i=0;i<args.length;i++)
	{
	    if(args[i].startsWith("-t="))
	    {
		try
		{
		    tabs = new Integer(args[i].substring(3)).intValue();
		}
		catch(Exception e) // No point in differentiating IndexOutOfBounds
		{
		    System.err.println("Incorrect format for -t parameter");
		    usage();
		}
	    }
	    else if(args[i].startsWith("-s="))
	    {
		try
		{
		    spaces = new Integer(args[i].substring(3)).intValue();
		}
		catch(Exception e)
		{
		    System.err.println("Incorrect format for -s parameter");
		    usage();
		}
	    }
	    else if(args[i].startsWith("-f="))
	    {
		try
		{
		    fill = new Integer(args[i].substring(3)).intValue();
		}
		catch(Exception e)
		{
		    System.err.println("Incorrect format for -f parameter");
		    usage();
		}
	    }
	    else if(args[i].startsWith("-a="))
	    {
		try
		{
		    attributes = new Integer(args[i].substring(3)).intValue();
		}
		catch(Exception e)
		{
		    System.err.println("Incorrect format for -a parameter");
		    usage();
		}
	    }
	    else if(args[i].startsWith("-parser="))
	    {
		try
		{
		    parserName = args[i].substring(8);
		}
		catch(Exception e)
		{
		    System.err.println("Incorrect format for -parser parameter");
		    usage();
		}
	    }
	    else if(args[i].equals("-l1"))
	    {
		alignFirstLine = true;
		alignText = true;
	    }
	    else if(args[i].startsWith("-l"))
	    {
		try
		{
		    if(args[i].length() > 2) //if an offset is specified with +X
			lineOffset = new Integer(args[i].substring(3)).intValue();
		}
		catch(Exception e)
		{
		    System.err.println("Incorrect format for -l parameter");
		    usage();
		}
	    }
	    else if(args[i].startsWith("-h"))
		usage();
	    else if(!args[i].startsWith("-") && in == null)
		in = args[i];
	    else if(!args[i].startsWith("-") && out == null)
		out = args[i];
	    else
		System.err.println("Unrecognized Option:"+args[i]);
	}
	startIndent();
    }

// ******************************************                              Usage
  /** 
      Simply output the usage information to the standard output.
   */
 
    public static void usage()
    {
	System.out.println("Usage:");

	System.out.println("java com.faa.sax.util.Indent [options] "+
                           "[inputFile [outputFile]]");
	System.out.println("Options:");

	System.out.println("-t=X indent elements using X tabs. Overrides " +
                           "-s.Default: off.");
	System.out.println("-s=X indent elements using X spaces.Default: -s=3");
	System.out.println("-f=X auto-fill text at column X.Default: off.");
	System.out.println("-a=X number of attributes per line.Default: -a=1");
	System.out.println("-l+X align running line text to start " +
                           "(plus and optional X offset). Default: -l+1");
	System.out.println("-l1 align first line of text too. Default: off");
	System.out.println("-parser=X. sets parser to class X.Default:" +
			   "org.apache.xerces.parsers.SAXParser");
	System.exit(1);
    }

// ******************************************                          Atributes

    static final String DEFAULT_PARSER = "org.apache.xerces.parsers.SAXParser";

    static int tabs = 0;
    static int spaces = 3;
    static int fill = 0;
    static int attributes = 1;
    static int lineOffset = 1;
    static String in = null;
    static String out = null;
    static String parserName = DEFAULT_PARSER;
    static boolean lineAlignedWithStart = true;
    static boolean alignFirstLine = false; 
    static boolean alignText = false;

}















